@@ -60,13 +60,23 @@ class Cvt_Int_Shutdown(DiffConvert_Int):
             return self.enter(int_name) + [" shutdown"]
 
 
-# we do VRF changes on an interface before we do any IP configuration,
-# otherwise it will be removed
+# converter to detect a VRF change and fire a trigger to make changes
+# required before it is actually changed
+
+class Cvt_VRFTrgr_VRFFwd(DiffConvert_Int):
+    cmd = "vrf-forwarding",
+    block = "int-vrf-trigger"
+    trigger_blocks = { "int-vrf-pre" }
+    empty_trigger = True
+
+
+# we do VRF changes on an interface before we do any IP address
+# configuration, otherwise the IP configuration will be removed
 
 class Cvt_Int_VRFFwd(DiffConvert_Int):
     cmd = "vrf-forwarding",
     block = "int-vrf"
-    trigger_blocks = { "int-addr" }
+    trigger_blocks = { "int-vrf-post" }
 
     def remove(self, old, int_name):
         return self.enter(int_name) + [" no vrf forwarding"]
@@ -146,7 +156,7 @@ class Cvt_Int_IPAccGrp(DiffConvert_Int):
 
 class Cvt_Int_IPAddr(DiffConvert_Int):
     cmd = "ip-address",
-    block = "int-addr"
+    block = "int-vrf-post"
 
     def remove(self, old, int_name):
         return self.enter(int_name) + [" no ip address"]
@@ -157,7 +167,7 @@ class Cvt_Int_IPAddr(DiffConvert_Int):
 
 class Cvt_Int_IPAddrSec(DiffConvert_Int):
     cmd = "ip-address-secondary", None
-    block = "int-addr"
+    block = "int-vrf-post"
 
     def remove(self, old, int_name, addr):
         return self.enter(int_name) + [" no ip address %s secondary" % addr]
@@ -166,6 +176,62 @@ class Cvt_Int_IPAddrSec(DiffConvert_Int):
         return self.enter(int_name) + [" ip address %s secondary" % addr]
 
 
+# ip helper-address ...
+#
+# This command is odd in that changing VRF causes it to be modified but
+# not removed (as configuration items mentioning IP addresses are).  For
+# example, if the interface is not in a VRF but is being moved into one,
+# the VRF change will cause it to change to 'ip helper-address global
+# ADDR'; removing it from a VRF change it to 'ip helper-address vrf
+# VRF-NAME ADDR'.
+#
+# To handle this, we remove the helper address before the VRF change and
+# then re-add it after changing it.  This is achieved through blocks and
+# triggers.
+
+
+class DiffConvert_Int_IPHlprAddr(DiffConvert_Int):
+    "Abstract class for 'ip helper-address'."
+
+    cmd = "ip-helper-address", None
+
+    def _cmd(self, helper):
+        return ("ip helper-address"
+                + (" global" if "global" in helper else "")
+                + (" vrf " + helper["vrf"] if "vrf" in helper else "")
+                + " " + helper["addr"])
+
+
+class Cvt_VRFPre_IPHlprAddr(DiffConvert_Int_IPHlprAddr):
+    """Class to handle removing 'ip helper-address' commands prior to a
+    VRF change.
+    """
+
+    block = "int-vrf-pre"
+
+    def remove(self, old, int_name, helper):
+        # if we're removing the helper address anyway, we just do that now
+        return self.enter(int_name) + [" no " + self._cmd(old)]
+
+    def trigger(self, new, int_name, helper):
+        # if we're not changing the helper address, we remove it prior
+        # to changing VRF (to be re-added after the VRF change)
+        return self.remove(new, int_name, helper)
+
+
+class Cvt_VRFPost_Int_IPHlprAddr(DiffConvert_Int_IPHlprAddr):
+    """Class to handle adding (or re-adding, post a VRF change) 'ip
+    helper-address' commands.
+    """
+
+    block = "int-vrf-post"
+
+    # removing helper addresses is done before the VRF changes
+
+    def update(self, old, upd, new, int_name, helper):
+        return self.enter(int_name) + [" " + self._cmd(new)]
+
+
 # ...
 
 
@@ -187,16 +253,6 @@ class Cvt_Int_IPFlowMon(DiffConvert_Int):
         return l
 
 
-class Cvt_Int_IPHlprAddr(DiffConvert_Int):
-    cmd = "ip-helper-address", None
-
-    def remove(self, old, int_name, addr):
-        return self.enter(int_name) + [" no ip helper-address " + addr]
-
-    def update(self, old, upd, new, int_name, addr):
-        return self.enter(int_name) + [" ip helper-address " + addr]
-
-
 class Cvt_Int_IPIGMPVer(DiffConvert_Int):
     cmd = "ip-igmp-version",
 
@@ -309,6 +365,7 @@ class Cvt_Int_IPPIMMode(DiffConvert_Int):
 
 class Cvt_Int_IPPIMBSRBdr(DiffConvert_Int):
     cmd = "ip-pim", "bsr-border"
+    block = "int-vrf-post"
 
     def remove(self, old, int_name):
         return self.enter(int_name) + [" no ip pim bsr-border"]
@@ -353,6 +410,7 @@ class Cvt_Int_IPVerifyUni(DiffConvert_Int):
 
 class Cvt_Int_IPv6Addr(DiffConvert_Int):
     cmd = "ipv6-address", None
+    block = "int-vrf-post"
 
     def remove(self, old, int_name, addr):
         return self.enter(int_name) + [" no ipv6 address " + addr]
@@ -373,6 +431,7 @@ class Cvt_Int_IPv6MultBdry(DiffConvert_Int):
 
 class Cvt_Int_IPv6PIMBSRBdr(DiffConvert_Int):
     cmd = "ipv6-pim", "bsr-border"
+    block = "int-vrf-post"
 
     def remove(self, old, int_name):
         return self.enter(int_name) + [" no ipv6 pim bsr border"]
@@ -533,7 +592,7 @@ class Cvt_Int_OSPFv3Net(DiffConvert_Int):
 
 class Cvt_Int_StandbyIP(DiffConvert_Int):
     cmd = "standby", "group", None, "ip"
-    block = "int-addr"
+    block = "int-vrf-post"
 
     def remove(self, old, int_name, grp):
         return self.enter(int_name) + [" no standby %d ip" % grp]
@@ -544,7 +603,7 @@ class Cvt_Int_StandbyIP(DiffConvert_Int):
 
 class Cvt_Int_StandbyIPSec(DiffConvert_Int):
     cmd = "standby", "group", None, "ip-secondary", None
-    block = "int-addr"
+    block = "int-vrf-post"
 
     def remove(self, old, int_name, grp, addr):
         return self.enter(int_name) + [
@@ -557,7 +616,7 @@ class Cvt_Int_StandbyIPSec(DiffConvert_Int):
 
 class Cvt_Int_StandbyIPv6(DiffConvert_Int):
     cmd = "standby", "group", None, "ipv6", None
-    block = "int-addr"
+    block = "int-vrf-post"
 
     def remove(self, old, int_name, grp, addr):
         return self.enter(int_name) + [" no standby %d ipv6 %s" % (grp, addr)]
